@@ -1,6 +1,5 @@
-use std::intrinsics::transmute;
 use std::num::Wrapping;
-use std::ops::{Add, AddAssign, BitXor, BitXorAssign, Index, IndexMut};
+use std::ops::{Add, BitXor, BitXorAssign, Index, IndexMut};
 
 #[derive(Clone, Copy, Debug)]
 pub struct ZestBlock {
@@ -21,24 +20,15 @@ impl ZestBlock {
             carry = (*elem < carry) as u64;
         }
     }
-    //Act on data as if it were u8 array.
+    //Present the data as big endian (for consistency)
+    //Discourage multiple calls as this has potentially unwanted overhead
     #[inline(always)]
-    pub fn data_u8_mut(&mut self) -> &mut [u8] {
-        unsafe {
-            transmute(std::slice::from_raw_parts_mut(
-                self.data.as_mut_ptr() as *mut u8,
-                self.data.len() * std::mem::size_of::<u64>(),
-            ))
-        }
-    }
-    #[inline(always)]
-    pub fn data_u8(&self) -> &[u8] {
-        unsafe {
-            transmute(std::slice::from_raw_parts(
-                self.data.as_ptr() as *const u8,
-                self.data.len() * std::mem::size_of::<u64>(),
-            ))
+    pub fn data_u8(&self) -> [u8; 512] {
+        let mut u8_data = [0_u8; 512];
+        for (i, chunk) in u8_data.chunks_exact_mut(8).enumerate() {
+            chunk.copy_from_slice(&self.data[i].to_be_bytes());
         }
+        u8_data
     }
 }
 
@@ -55,14 +45,6 @@ impl Add<ZestBlock> for ZestBlock {
         total
     }
 }
-impl AddAssign<ZestBlock> for ZestBlock {
-    #[inline(always)]
-    fn add_assign(&mut self, rhs: ZestBlock) {
-        for (index, chunk) in self.data.iter_mut().enumerate() {
-            *chunk = (Wrapping(*chunk) + Wrapping(rhs[index])).0;
-        }
-    }
-}
 
 //XOR, bitwise.
 impl BitXor<ZestBlock> for ZestBlock {
@@ -107,16 +89,18 @@ impl From<[u64; 64]> for ZestBlock {
         ZestBlock { data }
     }
 }
+
+//From big endian (for consistency)
+//Use [u64; u64] if possible, due to overhead of conversion
 impl From<[u8; 512]> for ZestBlock {
     #[inline(always)]
     fn from(data: [u8; 512]) -> Self {
-        let u64_ptr = data.as_ptr() as *const u64;
-        // Safety: Cast using same number of bytes.
-        let owned_slice = unsafe { std::slice::from_raw_parts(u64_ptr, 64) };
-        let converted_data = owned_slice.try_into().unwrap();
-        ZestBlock {
-            data: converted_data,
+        // Safety: All values of array are initialized.
+        let mut u64_data = [0_u64; 64];
+        for (i, chunk) in data.chunks_exact(8).enumerate() {
+            u64_data[i] = u64::from_be_bytes(chunk.try_into().unwrap());
         }
+        ZestBlock { data: u64_data }
     }
 }
 
